
"""

Module zttt
============

The module contains 2 main classes
Both the classes inherit from the ZTBaseBoard class.

1. PvP: The class for the PvP game
2. PvC: The class for the PvC game


The module also contains the submodule zt_errors which contain the following errors

1. ZTError: The base class for all ZTErrors
2. ZTBadFunctionCall: The error raised when a function is called with the wrong arguments. Mostly used for debugging
3. ZTGameException: The error raised when a function call crashes the game
4. ZTWrongInput: The error raised when the input provided is not valid

"""

from .pvc import PvC
from .pvp import PvP
from . import zt_errors

__version__ = '1.0.1'

__all__ = ['__version__', 'PvP', 'PvC', 'zt_errors']



from ._zt_core import ZTBaseBoard


class PvP(ZTBaseBoard):

    def __init__(self) -> None:
        """Initialize the Game"""
        super().__init__()

    def play(self, pos: int) -> None:
        """Plays the player's move at the position specified

        :param pos: The position to play the player's move
        :type pos: int
        :return: None
        :raise: ZTGameException if the game is not in progress
        :raise: ZTInvalidInput if the move is invalid
        """

        if self.turn == 1:
            super()._play_player_one_move(pos)
        else:
            super()._play_player_two_move(pos)




"""
_zt_core module
-----------------

The module contains the core base classes for the zttt package.
"""


from .zt_base_board import ZTBaseBoard
from .zt_engine_first import ZTEngineFirst
from .zt_player_first import ZTPlayerFirst



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zttt', 'zttt._zt_core']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'zttt',
    'version': '1.0.1',
    'description': 'Simple, Standalone Tic Tac Toe Library',
    'long_description': "\n.. _zttt_main:\n\nZTicTacToe\n============\n\nTic Tac Toe is a famous game in which two players take turns placing\na mark on a 3x3 grid. The first player to get three in a row wins.\n\nThe module is a standalone implementation of the game TicTacToe\nproviding functionality to keep track of the game's state and to\nmake moves.\n\n\n.. _zttt_features:\n\nFeatures\n---------\n- Standalone implementation of the game Tic Tac Toe.\n- Provides a way to customise **move triggers** and access state variables.\n- Comes with an engine with near perfect moves.\n- Written in Python from scratch and does not require any external libraries.\n- Can be integrated into a larger project, with very little effort.\n- Throws custom-built errors making it easy to debug and handle errors.\n\n\n\n.. _zttt_links:\n\nLinks\n------\n- `PyPI <https://pypi.python.org/pypi/zttt>`_\n- `GitHub <https://github.com/Sigma1084/ZTicTacToe/tree/v1>`_\n- `Documentation <https://ztictactoe.readthedocs.io/en/v1/>`_\n- `Examples <https://github.com/Sigma1084/ZTicTacToe/tree/v1/examples>`_\n\n.. _zttt_install:\n\nInstallation\n-------------\n.. code-block:: bash\n\n    pip install zttt\n",
    'author': 'Sumanth NR',
    'author_email': 'sumanthnr62@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://ztictactoe.readthedocs.io/en/latest/',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from ._zt_core import ZTEngineFirst
from ._zt_core import ZTPlayerFirst


class PvC(ZTEngineFirst, ZTPlayerFirst):
    """Class for the PvC Game"""

    def __init__(self, _engine_first: bool = True) -> None:
        """Initialize the Game

        :param _engine_first: Specifies if the engine starts first
        :type _engine_first: bool
        :return: None
        """

        if _engine_first:
            self.parent = ZTEngineFirst
            ZTEngineFirst.__init__(self)

        else:
            self.parent = ZTPlayerFirst
            ZTPlayerFirst.__init__(self)

    def play(self, pos: int) -> None:
        """Plays the player's move at the position specified

        :param pos: The position to play the player's move
        :type pos: int
        :return: None
        :raise: ZTGameException if the game is not in progress
        :raise: ZTInvalidInput if the move is invalid
        """

        self.parent.play(self, pos)




"""
Module zttt.zt_errors
======================

The module contains the errors that can be raised by the zttt package.
"""


class ZTError(Exception):
    """Base class for all zt_errors."""
    pass


class ZTBadFunctionCall(AttributeError, ZTError):
    """Raised when a function is called with the wrong arguments. Mostly used for debugging."""
    pass


class ZTGameException(ZTError):
    """Raised when some invalid function call crashes the game."""
    pass


class ZTInvalidInput(ValueError, ZTGameException):
    """Raised when the input provided is not valid."""
    pass


__all__ = ['ZTError', 'ZTBadFunctionCall', 'ZTGameException', 'ZTInvalidInput']
